/*
 *  Copyright (c) 1998-2001, The University of Sheffield.
 *
 *  This file is part of GATE (see http://gate.ac.uk/), and is free
 *  software, licenced under the GNU Library General Public License,
 *  Version 2, June 1991 (in the distribution as file licence.html,
 *  and also available at http://gate.ac.uk/gate/licence.html).
 *
 *  Valentin Tablan 26/10/2001
 *
 *  $Id: CollectionPersistence.java,v 1.3 2002/05/13 11:35:32 valyt Exp $
 *
 */
package gate.util.persistence;

import java.util.*;
import java.io.*;

import gate.persist.PersistenceException;
import gate.creole.ResourceInstantiationException;
import gate.util.*;


public class CollectionPersistence implements Persistence {

  /**
   * Populates this Persistence with the data that needs to be stored from the
   * original source object.
   */
  public void extractDataFromSource(Object source)throws PersistenceException{
    if(! (source instanceof Collection)){
      throw new UnsupportedOperationException(
                getClass().getName() + " can only be used for " +
                Collection.class.getName() +
                " objects!\n" + source.getClass().getName() +
                " is not a " + Collection.class.getName());
    }
    collectionType = source.getClass();

    Collection coll = (Collection)source;

    //get the values in the iterator's order
    localList = new ArrayList(coll.size());
    Iterator elemIter = coll.iterator();
    while(elemIter.hasNext()){
      localList.add(PersistenceManager.
                    getPersistentRepresentation(elemIter.next()));
    }
  }

  /**
   * Creates a new object from the data contained. This new object is supposed
   * to be a copy for the original object used as source for data extraction.
   */
  public Object createObject()throws PersistenceException,
                                     ResourceInstantiationException{
    //let's try to create a collection of the same type as the original
    Collection result = null;
    try{
      result = (Collection)collectionType.newInstance();
    }catch(Exception e){
    }
    if(result == null) result = new ArrayList(localList.size());

    //now we have the collection let's populate it
    Iterator elemIter = localList.iterator();
    while(elemIter.hasNext()){
      try{
        result.add(PersistenceManager.
                   getTransientRepresentation(elemIter.next()));
      }catch(PersistenceException pe){
        PersistenceManager.exceptionOccured = true;
        pe.printStackTrace(Err.getPrintWriter());
      }catch(ResourceInstantiationException rie){
        PersistenceManager.exceptionOccured = true;
        rie.printStackTrace(Err.getPrintWriter());
      }
    }

    return result;


  }


  protected List localList;
  protected Class collectionType;
  static final long serialVersionUID = 7908364068699089834L;
}